use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;

use crate::{
    endpoints::assignment::PlanEventPersonalTablePartialTemplate,
    utils::{
        event_planning_template::{
            generate_availability_assignment_list, generate_status_whether_staff_is_required,
        },
        ApplicationError, TemplateResponse,
    },
};

use brass_db::{
    models::{Assignment, AssignmentChangeset, AssignmentContext, Event, Function, User},
    validation::AsyncValidate,
};

#[derive(Deserialize)]
pub struct AssignmentQuery {
    availability: i32,
    function: u8,
    event: i32,
}

#[actix_web::post("/assignments/new")]
pub async fn post(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    query: web::Query<AssignmentQuery>,
) -> Result<impl Responder, ApplicationError> {
    let Some(event) = Event::read_by_id_including_location(pool.get_ref(), query.event).await?
    else {
        return Ok(HttpResponse::NotFound().finish());
    };

    let function = Function::try_from(query.function)?;

    let changeset = AssignmentChangeset {
        function,
        time: (event.start, event.end),
    };

    let context = AssignmentContext {
        user: &user.into_inner(),
        event_id: event.id,
        availability_id: query.availability,
        pool: pool.get_ref(),
    };

    if let Err(e) = changeset.validate_with_context(&context).await {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    Assignment::create(pool.get_ref(), event.id, query.availability, changeset).await?;

    let availabilities = generate_availability_assignment_list(pool.get_ref(), &event).await?;

    let (
        further_posten_required,
        further_fuehrungsassistent_required,
        further_wachhabender_required,
    ) = generate_status_whether_staff_is_required(pool.get_ref(), &event).await?;

    let template = PlanEventPersonalTablePartialTemplate {
        event,
        availabilities,
        further_posten_required,
        further_fuehrungsassistent_required,
        further_wachhabender_required,
    };

    Ok(template.to_response()?)
}
